"""Create a client to send packages to the Vista Gateway API."""

from typing import Optional

from vista_gateway_client.gateway_package import GatewayPackage
from vista_gateway_client.implementation.http_client import HttpClientImpl
# from vista_gateway_client.implementation.mqtt_client import MqttClientImpl
from .vista_gateway_client import VistaGatewayClient
from vista_gateway_client.vista_repository_environment import (
    VistaRepositoryEnvironmentHttp,
    VistaRepositoryEnvironmentMqtt,
)


class Client:
    """Client for sending packages to the Vista Gateway API."""

    def __init__(
        self,
        api_key: Optional[str],
        protocol: str = "Http",
        environment: str = "Production",
    ) -> None:
        """Initialize the client with the API key, protocol, and environment."""
        self._api_key = api_key
        self._protocol = protocol
        self._environment = environment
        self._client = self.add_vista_gateway_client(api_key, protocol, environment)

    def add_vista_gateway_client(
        self,
        api_key: Optional[str],
        protocol: str,
        environment: str,
    ) -> VistaGatewayClient:
        """Initialize the Vista Gateway client based on the protocol."""        
        if not api_key:
            raise ValueError("API key is required")

        if protocol == "Mqtt":
            endpoint = VistaRepositoryEnvironmentMqtt[environment]
            # return MqttClientImpl(api_key, endpoint.value)
            raise NotImplementedError(f"Protocol {protocol} is not supported")
        elif protocol == "Http":
            endpoint = VistaRepositoryEnvironmentHttp[environment]            
            return HttpClientImpl(api_key, endpoint.value)
        else:
            raise NotImplementedError(f"Protocol {protocol} is not supported")
        
    def send(
        self,
        package: GatewayPackage,
        # cancellationToken: CancellationToken = default,
    ) -> None:
        return self._client.send(package)
