"""GatewayPackage and ISO19848 classes for Vista Gateway Python client."""

from abc import ABC
from dataclasses import dataclass
from typing import Optional
from jsonschema import validate

import vista_gateway_client.resources.data_channel as DataChannel
import vista_gateway_client.resources.time_series as TimeSeries
from vista_gateway_client.resources.data_channel import schema_data_channel
from vista_gateway_client.resources.time_series import schema_time_series


class GatewayPackage(ABC):
    """Abstract base class for GatewayPackage."""

    def to_json_dto(self) -> str:
        """Convert the package to a JSON string."""
        # return {"Package": self.package.to_json()}
        return '{"Package": ' + self.package.serialize() + '}'


class ISO19848:
    """Container for GatewayPackage classes."""

    @dataclass(frozen=True)
    class DataChannelList(GatewayPackage):
        """DataChannelList package for Vista Gateway."""

        package: DataChannel.Package
        external_id: Optional[str] = None
        correlation_id: Optional[str] = None
        user_agent: Optional[str] = None

        def __post_init__(self):
            if not isinstance(self.package, DataChannel.Package):
                raise TypeError("package is not of type DataChannel.Package")
            validate({"Package": self.package.for_json()}, schema_data_channel)

    @dataclass(frozen=True)
    class TimeSeriesData(GatewayPackage):
        """TimeSeriesData package for Vista Gateway."""

        package: TimeSeries.Package
        external_id: Optional[str] = None
        correlation_id: Optional[str] = None
        user_agent: Optional[str] = None

        def __post_init__(self):
            if not isinstance(self.package, TimeSeries.Package):
                raise TypeError("package is not of type TimeSeries.Package")
            validate({"Package": self.package.for_json()}, schema_time_series)
