"""Define exceptions that can be raised by the Vista Gateway Client."""

from http import HTTPStatus
from typing import Optional


class VistaGatewayException(Exception):
    """Represents an exception that occurred in the Vista Gateway Client."""

    def __init__(
        self,
        message: str,
        status: Optional[HTTPStatus] = None,
        reason: Optional[str] = None,
        inner_exception: Optional[Exception] = None,
    ) -> None:
        """Initialize a new instance of the VistaGatewayException class."""
        self.status = str(status) if status is not None else None
        self.reason = reason
        super().__init__(f"{message}: Status={self.status}, Reason={self.reason}")
        self.__cause__ = inner_exception
